from __future__ import print_function
from __future__ import absolute_import
from builtins import zip
from builtins import str
from builtins import object
from future.utils import raise_from

import sys
import yaml
import struct
import snappy
import traceback
from functools import partial

from .mapping import NAME_CLASS_MAP, ID_NAME_MAP

from google.protobuf.internal.encoder import _VarintBytes
from google.protobuf.internal.decoder import _DecodeVarint32
from google.protobuf.json_format import MessageToDict, ParseDict
from google.protobuf.message import EncodeError

from .generated.TSPArchiveMessages_pb2 import ArchiveInfo


class IWAFile(object):
    def __init__(self, chunks, filename=None):
        self.chunks = chunks
        self.filename = filename

    @classmethod
    def from_file(cls, filename):
        with open(filename) as f:
            return cls.from_buffer(f.read(), filename)

    @classmethod
    def from_buffer(cls, data, filename=None):
        try:
            chunks = []
            while data:
                chunk, data = IWACompressedChunk.from_buffer(data, filename)
                chunks.append(chunk)

            return cls(chunks, filename)
        except Exception as e:
            if filename:
                raise_from(ValueError("Failed to deserialize " + filename), e)
            else:
                raise

    @classmethod
    def from_dict(cls, _dict):
        return cls([IWACompressedChunk.from_dict(chunk) for chunk in _dict['chunks']])

    def to_dict(self):
        try:
            return {"chunks": [chunk.to_dict() for chunk in self.chunks]}
        except Exception as e:
            if self.filename:
                raise_from(ValueError("Failed to serialize " + self.filename), e)
            else:
                raise

    def to_buffer(self):
        return b''.join([chunk.to_buffer() for chunk in self.chunks])


class IWACompressedChunk(object):
    def __init__(self, archives):
        self.archives = archives

    def __eq__(self, other):
        return self.archives == other.archives

    @classmethod
    def _decompress_all(cls, data):
        while data:
            header = data[:4]

            first_byte = header[0]
            if not isinstance(first_byte, int):
                first_byte = ord(first_byte)

            if first_byte != 0x00:
                raise ValueError("IWA chunk does not start with 0x00! (found %x)" % first_byte)

            unpacked = struct.unpack_from('<I', bytes(header[1:]) + b'\x00')
            length = unpacked[0]
            chunk = data[4 : 4 + length]
            data = data[4 + length :]

            try:
                yield snappy.uncompress(chunk)
            except Exception:
                # Try to see if this data isn't compressed in the first place.
                # If this data is still compressed, parsing it as Protobuf
                # will almost definitely fail anyways.
                yield chunk

    @classmethod
    def from_buffer(cls, data, filename=None):
        data = b''.join(cls._decompress_all(data))
        archives = []
        while data:
            archive, data = IWAArchiveSegment.from_buffer(data, filename)
            archives.append(archive)
        return cls(archives), None

    @classmethod
    def from_dict(cls, _dict):
        return cls([IWAArchiveSegment.from_dict(d) for d in _dict['archives']])

    def to_dict(self):
        return {"archives": [archive.to_dict() for archive in self.archives]}

    def to_buffer(self):
        uncompressed = b''.join([archive.to_buffer() for archive in self.archives])
        payloads = []
        while uncompressed:
            payloads.append(snappy.compress(uncompressed[:65536]))
            uncompressed = uncompressed[65536:]
        return b''.join(
            [b'\x00' + struct.pack('<I', len(payload))[:3] + payload for payload in payloads]
        )


class ProtobufPatch(object):
    def __init__(self, data):
        self.data = data

    def __eq__(self, other):
        return self.data == other.data

    def __repr__(self):
        return "<%s %s>" % (self.__class__.__name__, self.data)

    def to_dict(self):
        return message_to_dict(self.data)

    @classmethod
    def FromString(cls, message_info, proto_klass, data):
        if len(message_info.diff_field_path.path) != 1:
            raise NotImplementedError(
                "Not sure how to deserialize ProtobufPatch without exactly one diff_field_path. "
                "Message info was:\n%s\nObject was:\n%s" % (message_info, data)
            )
        if message_info.fields_to_remove:
            raise NotImplementedError(
                "Not sure how to deserialize ProtobufPatch with fields_to_remove. "
                "Message info was:\n%s\nObject was:\n%s" % (message_info, data)
            )
        for diff_path in message_info.diff_field_path.path:
            patched_field = proto_klass.DESCRIPTOR.fields_by_number[diff_path]
            field_message_class = NAME_CLASS_MAP[patched_field.message_type.full_name]
            return cls(field_message_class.FromString(data))

    def SerializeToString(self):
        return self.data.SerializePartialToString()


class IWAArchiveSegment(object):
    def __init__(self, header, objects):
        self.header = header
        self.objects = objects

    def __eq__(self, other):
        return self.header == other.header and self.objects == other.objects

    def __repr__(self):
        return "<%s identifier=%s objects=%s>" % (
            self.__class__.__name__,
            self.header.identifier,
            repr(self.objects).replace("\n", " ").replace("  ", " "),
        )

    @classmethod
    def from_buffer(cls, buf, filename=None):
        archive_info, payload = get_archive_info_and_remainder(buf)
        if not repr(archive_info):
            raise ValueError("Segment doesn't seem to start with an ArchiveInfo!")

        payloads = []

        n = 0
        for message_info in archive_info.message_infos:
            try:
                if message_info.type == 0 and archive_info.should_merge and payloads:
                    base_message = archive_info.message_infos[message_info.base_message_index]
                    klass = partial(
                        ProtobufPatch.FromString, message_info, ID_NAME_MAP[base_message.type]
                    )
                else:
                    klass = ID_NAME_MAP[message_info.type]
            except KeyError:
                raise NotImplementedError(
                    "Don't know how to parse Protobuf message type " + str(message_info.type)
                )
            try:
                message_payload = payload[n : n + message_info.length]
                if hasattr(klass, 'FromString'):
                    output = klass.FromString(message_payload)
                else:
                    output = klass(message_payload)
            except Exception as e:
                raise ValueError(
                    "Failed to deserialize %s payload of length %d: %s"
                    % (klass, message_info.length, e)
                )
            payloads.append(output)
            n += message_info.length

        return cls(archive_info, payloads), payload[n:]

    @classmethod
    def from_dict(cls, _dict):
        header = dict_to_header(_dict['header'])
        objects = []
        for message_info, o in zip(header.message_infos, _dict['objects']):
            if message_info.diff_field_path and '_pbtype' not in o:
                base_message_info = header.message_infos[message_info.base_message_index]
                message_class = ID_NAME_MAP[base_message_info.type]
                objects.append(ProtobufPatch(message_class, o))
            else:
                objects.append(dict_to_message(o))
        return cls(header, objects)

    def to_dict(self):
        return {
            "header": header_to_dict(self.header),
            "objects": [message_to_dict(message) for message in self.objects],
        }

    def to_buffer(self):
        # Each message_info as part of the header needs to be updated
        # so that its length matches the object contained within.
        for obj, message_info in zip(self.objects, self.header.message_infos):
            try:
                object_length = len(obj.SerializeToString())
                provided_length = message_info.length
                if object_length != provided_length:
                    message_info.length = object_length
            except EncodeError as e:
                raise ValueError(
                    "Failed to encode object: %s\nObject: '%s'\nMessage info: %s"
                    % (e, repr(obj), message_info)
                )
        return b''.join(
            [_VarintBytes(self.header.ByteSize()), self.header.SerializeToString()]
            + [obj.SerializeToString() for obj in self.objects]
        )


def message_to_dict(message):
    if hasattr(message, 'to_dict'):
        return message.to_dict()
    output = MessageToDict(message)
    output['_pbtype'] = type(message).DESCRIPTOR.full_name
    return output


def header_to_dict(message):
    output = message_to_dict(message)
    for message_info in output['messageInfos']:
        del message_info['length']
    return output


def dict_to_message(_dict):
    _type = _dict['_pbtype']
    del _dict['_pbtype']
    return ParseDict(_dict, NAME_CLASS_MAP[_type](), ignore_unknown_fields=True)


def dict_to_header(_dict):
    for message_info in _dict['messageInfos']:
        # set a dummy length value that we'll overwrite later
        message_info['length'] = 0
    return dict_to_message(_dict)


def get_archive_info_and_remainder(buf):
    msg_len, new_pos = _DecodeVarint32(buf, 0)
    n = new_pos
    msg_buf = buf[n : n + msg_len]
    n += msg_len
    return ArchiveInfo.FromString(msg_buf), buf[n:]


if __name__ == "__main__":
    for filename in sys.argv[1:]:
        try:
            iwa_file = IWAFile.from_file(filename)
            print(yaml.safe_dump(iwa_file.to_dict(), default_flow_style=False))
        except Exception as e:
            print("FAILED", traceback.format_exc(e))



from __future__ import absolute_import
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: TSCH3DArchives.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import TSPMessages_pb2 as TSPMessages__pb2
from . import TSDArchives_pb2 as TSDArchives__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='TSCH3DArchives.proto',
  package='TSCH',
  syntax='proto2',
  serialized_options=None,
  serialized_pb=_b('\n\x14TSCH3DArchives.proto\x12\x04TSCH\x1a\x11TSPMessages.proto\x1a\x11TSDArchives.proto\"^\n Chart3DEnvironmentPackageArchive\x12:\n\tmaterials\x18\x01 \x03(\x0b\x32\'.TSCH.Chart3DEnvironmentMaterialArchive\"\xa6\x01\n\x12\x43hart3DFillArchive\x12\x38\n\rlightingmodel\x18\x01 \x01(\x0b\x32!.TSCH.Chart3DLightingModelArchive\x12\x15\n\rtextureset_id\x18\x02 \x01(\t\x12)\n\tfill_type\x18\x03 \x01(\x0e\x32\x16.TSCH.FillPropertyType\x12\x14\n\x0cseries_index\x18\x04 \x01(\r\"H\n\x18\x43hart3DPointLightArchive\x12,\n\x08position\x18\x01 \x02(\x0b\x32\x1a.TSCH.Chart3DVectorArchive\"O\n\x1e\x43hart3DDirectionalLightArchive\x12-\n\tdirection\x18\x01 \x02(\x0b\x32\x1a.TSCH.Chart3DVectorArchive\"\x97\x01\n\x17\x43hart3DSpotLightArchive\x12,\n\x08position\x18\x01 \x02(\x0b\x32\x1a.TSCH.Chart3DVectorArchive\x12-\n\tdirection\x18\x02 \x02(\x0b\x32\x1a.TSCH.Chart3DVectorArchive\x12\x0e\n\x06\x63utoff\x18\x03 \x02(\x02\x12\x0f\n\x07\x64ropoff\x18\x04 \x02(\x02\"\xd5\x03\n\x13\x43hart3DLightArchive\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x31\n\rambient_color\x18\x02 \x02(\x0b\x32\x1a.TSCH.Chart3DVectorArchive\x12\x31\n\rdiffuse_color\x18\x03 \x02(\x0b\x32\x1a.TSCH.Chart3DVectorArchive\x12\x32\n\x0especular_color\x18\x04 \x02(\x0b\x32\x1a.TSCH.Chart3DVectorArchive\x12\x11\n\tintensity\x18\x05 \x02(\x02\x12/\n\x0b\x61ttenuation\x18\x06 \x02(\x0b\x32\x1a.TSCH.Chart3DVectorArchive\x12\x18\n\x10\x63oordinate_space\x18\x07 \x02(\r\x12\x0f\n\x07\x65nabled\x18\x08 \x02(\x08\x12\x33\n\x0bpoint_light\x18\t \x01(\x0b\x32\x1e.TSCH.Chart3DPointLightArchive\x12?\n\x11\x64irectional_light\x18\n \x01(\x0b\x32$.TSCH.Chart3DDirectionalLightArchive\x12\x31\n\nspot_light\x18\x0b \x01(\x0b\x32\x1d.TSCH.Chart3DSpotLightArchive\"\xd9\x01\n\x1b\x43hart3DLightingModelArchive\x12\x35\n\x05phong\x18\x01 \x01(\x0b\x32&.TSCH.Chart3DPhongLightingModelArchive\x12\x46\n\x0e\x66ixed_function\x18\x02 \x01(\x0b\x32..TSCH.Chart3DFixedFunctionLightingModelArchive\x12;\n\x0b\x65nvironment\x18\x03 \x01(\x0b\x32&.TSCH.Chart3DEnvironmentPackageArchive\"X\n\x1d\x43hart3DLightingPackageArchive\x12\x0c\n\x04name\x18\x01 \x02(\t\x12)\n\x06lights\x18\x02 \x03(\x0b\x32\x19.TSCH.Chart3DLightArchive\"\x86\x01\n\x1e\x43hart3DTexturesMaterialArchive\x12)\n\x05\x63olor\x18\x01 \x02(\x0b\x32\x1a.TSCH.Chart3DVectorArchive\x12\x39\n\x08textures\x18\x02 \x03(\x0b\x32\'.TSCH.Chart3DTSPImageDataTextureArchive\"\x8e\x01\n\x1e\x43hart3DEmissiveMaterialArchive\x12\x33\n\x05super\x18\x01 \x02(\x0b\x32$.TSCH.Chart3DTexturesMaterialArchive\x12\x37\n\x07tilings\x18\x02 \x03(\x0b\x32&.TSCH.Chart3DImageTextureTilingArchive\"\x8d\x01\n\x1d\x43hart3DDiffuseMaterialArchive\x12\x33\n\x05super\x18\x01 \x02(\x0b\x32$.TSCH.Chart3DTexturesMaterialArchive\x12\x37\n\x07tilings\x18\x02 \x03(\x0b\x32&.TSCH.Chart3DImageTextureTilingArchive\"\x8e\x01\n\x1e\x43hart3DModulateMaterialArchive\x12\x33\n\x05super\x18\x01 \x02(\x0b\x32$.TSCH.Chart3DTexturesMaterialArchive\x12\x37\n\x07tilings\x18\x02 \x03(\x0b\x32&.TSCH.Chart3DImageTextureTilingArchive\"\x8e\x01\n\x1e\x43hart3DSpecularMaterialArchive\x12\x33\n\x05super\x18\x01 \x02(\x0b\x32$.TSCH.Chart3DTexturesMaterialArchive\x12\x37\n\x07tilings\x18\x02 \x03(\x0b\x32&.TSCH.Chart3DImageTextureTilingArchive\"\x8f\x01\n\x1f\x43hart3DShininessMaterialArchive\x12\x33\n\x05super\x18\x01 \x02(\x0b\x32$.TSCH.Chart3DTexturesMaterialArchive\x12\x37\n\x07tilings\x18\x02 \x03(\x0b\x32&.TSCH.Chart3DImageTextureTilingArchive\"\xea\x01\n!Chart3DEnvironmentMaterialArchive\x12\x33\n\x05super\x18\x01 \x02(\x0b\x32$.TSCH.Chart3DTexturesMaterialArchive\x12\x44\n\x10OBSOLETE_tilings\x18\x02 \x03(\x0b\x32*.TSCH.Chart3DBaseImageTextureTilingArchive\x12\x11\n\tdecalMode\x18\x03 \x01(\x08\x12\x37\n\x07tilings\x18\x04 \x03(\x0b\x32&.TSCH.Chart3DImageTextureTilingArchive\"g\n(Chart3DFixedFunctionLightingModelArchive\x12;\n\tmaterials\x18\x01 \x02(\x0b\x32(.TSCH.Chart3DPhongMaterialPackageArchive\"_\n Chart3DPhongLightingModelArchive\x12;\n\tmaterials\x18\x01 \x02(\x0b\x32(.TSCH.Chart3DPhongMaterialPackageArchive\"\xbc\x02\n\"Chart3DPhongMaterialPackageArchive\x12\x36\n\x08\x65missive\x18\x01 \x01(\x0b\x32$.TSCH.Chart3DEmissiveMaterialArchive\x12\x34\n\x07\x64iffuse\x18\x02 \x01(\x0b\x32#.TSCH.Chart3DDiffuseMaterialArchive\x12\x36\n\x08modulate\x18\x03 \x01(\x0b\x32$.TSCH.Chart3DModulateMaterialArchive\x12\x36\n\x08specular\x18\x04 \x01(\x0b\x32$.TSCH.Chart3DSpecularMaterialArchive\x12\x38\n\tshininess\x18\x05 \x01(\x0b\x32%.TSCH.Chart3DShininessMaterialArchive\"\xc1\x01\n!Chart3DTSPImageDataTextureArchive\x12 \n\x04\x64\x61ta\x18\x03 \x01(\x0b\x32\x12.TSP.DataReference\x12&\n\nmipmapdata\x18\x04 \x01(\x0b\x32\x12.TSP.DataReference\x12%\n\rdatabase_data\x18\x01 \x01(\x0b\x32\x0e.TSP.Reference\x12+\n\x13\x64\x61tabase_mipmapdata\x18\x02 \x01(\x0b\x32\x0e.TSP.Reference\"c\n$Chart3DBaseImageTextureTilingArchive\x12)\n\x05scale\x18\x01 \x01(\x0b\x32\x1a.TSCH.Chart3DVectorArchive\x12\x10\n\x08rotation\x18\x02 \x01(\x02\"\xac\x03\n Chart3DImageTextureTilingArchive\x12\x39\n\x05super\x18\x01 \x02(\x0b\x32*.TSCH.Chart3DBaseImageTextureTilingArchive\x12%\n\x04mode\x18\x02 \x01(\x0e\x32\x17.TSCH.TextureTilingMode\x12%\n\x04wrap\x18\x03 \x01(\x0e\x32\x17.TSCH.TextureTilingWrap\x12%\n\x04\x66\x61\x63\x65\x18\x04 \x01(\x0e\x32\x17.TSCH.TextureTilingFace\x12/\n\txposition\x18\x05 \x01(\x0e\x32\x1c.TSCH.TextureTilingXPosition\x12/\n\typosition\x18\x06 \x01(\x0e\x32\x1c.TSCH.TextureTilingYPosition\x12\x32\n\x0bscontinuity\x18\x07 \x01(\x0e\x32\x1d.TSCH.TextureTilingContinuity\x12\x32\n\x0btcontinuity\x18\x08 \x01(\x0e\x32\x1d.TSCH.TextureTilingContinuity\x12\x0e\n\x06reveal\x18\t \x01(\x08\"B\n\x14\x43hart3DVectorArchive\x12\t\n\x01x\x18\x01 \x02(\x02\x12\t\n\x01y\x18\x02 \x02(\x02\x12\t\n\x01z\x18\x03 \x02(\x02\x12\t\n\x01w\x18\x04 \x02(\x02*L\n\x11TextureTilingMode\x12\x19\n\x15textureTilingModeNone\x10\x00\x12\x1c\n\x18textureTilingModeTallest\x10\x01*k\n\x11TextureTilingFace\x12\x18\n\x14textureTilingFaceAll\x10\x00\x12!\n\x1dtextureTilingFaceTopAndBottom\x10\x01\x12\x19\n\x15textureTilingFaceSide\x10\x02*R\n\x11TextureTilingWrap\x12\x1e\n\x1atextureTilingWrapProjected\x10\x00\x12\x1d\n\x19textureTilingWrapFaceWrap\x10\x01*{\n\x16TextureTilingXPosition\x12\x1e\n\x1atextureTilingXPositionLeft\x10\x00\x12 \n\x1ctextureTilingXPositionCenter\x10\x01\x12\x1f\n\x1btextureTilingXPositionRight\x10\x02*{\n\x16TextureTilingYPosition\x12\x1d\n\x19textureTilingYPositionTop\x10\x00\x12 \n\x1ctextureTilingYPositionMiddle\x10\x01\x12 \n\x1ctextureTilingYPositionBottom\x10\x02*\xa5\x01\n\x17TextureTilingContinuity\x12\x1f\n\x1btextureTilingContinuityNone\x10\x00\x12!\n\x1dtextureTilingContinuityGlobal\x10\x01\x12!\n\x1dtextureTilingContinuitySeries\x10\x02\x12#\n\x1ftextureTilingContinuityJittered\x10\x03*\xb3\x01\n\x10\x46illPropertyType\x12\x1d\n\x19\x66illPropertyTypeUndefined\x10\x00\x12\x18\n\x14\x66illPropertyTypeArea\x10\x01\x12\x17\n\x13\x66illPropertyTypeBar\x10\x02\x12\x1a\n\x16\x66illPropertyTypeColumn\x10\x03\x12\x18\n\x14\x66illPropertyTypeLine\x10\x04\x12\x17\n\x13\x66illPropertyTypePie\x10\x05::\n\x06\x66ill3d\x12\x10.TSD.FillArchive\x18\x64 \x01(\x0b\x32\x18.TSCH.Chart3DFillArchive')
  ,
  dependencies=[TSPMessages__pb2.DESCRIPTOR,TSDArchives__pb2.DESCRIPTOR,])

_TEXTURETILINGMODE = _descriptor.EnumDescriptor(
  name='TextureTilingMode',
  full_name='TSCH.TextureTilingMode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='textureTilingModeNone', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='textureTilingModeTallest', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3840,
  serialized_end=3916,
)
_sym_db.RegisterEnumDescriptor(_TEXTURETILINGMODE)

TextureTilingMode = enum_type_wrapper.EnumTypeWrapper(_TEXTURETILINGMODE)
_TEXTURETILINGFACE = _descriptor.EnumDescriptor(
  name='TextureTilingFace',
  full_name='TSCH.TextureTilingFace',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='textureTilingFaceAll', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='textureTilingFaceTopAndBottom', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='textureTilingFaceSide', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3918,
  serialized_end=4025,
)
_sym_db.RegisterEnumDescriptor(_TEXTURETILINGFACE)

TextureTilingFace = enum_type_wrapper.EnumTypeWrapper(_TEXTURETILINGFACE)
_TEXTURETILINGWRAP = _descriptor.EnumDescriptor(
  name='TextureTilingWrap',
  full_name='TSCH.TextureTilingWrap',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='textureTilingWrapProjected', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='textureTilingWrapFaceWrap', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=4027,
  serialized_end=4109,
)
_sym_db.RegisterEnumDescriptor(_TEXTURETILINGWRAP)

TextureTilingWrap = enum_type_wrapper.EnumTypeWrapper(_TEXTURETILINGWRAP)
_TEXTURETILINGXPOSITION = _descriptor.EnumDescriptor(
  name='TextureTilingXPosition',
  full_name='TSCH.TextureTilingXPosition',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='textureTilingXPositionLeft', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='textureTilingXPositionCenter', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='textureTilingXPositionRight', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=4111,
  serialized_end=4234,
)
_sym_db.RegisterEnumDescriptor(_TEXTURETILINGXPOSITION)

TextureTilingXPosition = enum_type_wrapper.EnumTypeWrapper(_TEXTURETILINGXPOSITION)
_TEXTURETILINGYPOSITION = _descriptor.EnumDescriptor(
  name='TextureTilingYPosition',
  full_name='TSCH.TextureTilingYPosition',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='textureTilingYPositionTop', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='textureTilingYPositionMiddle', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='textureTilingYPositionBottom', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=4236,
  serialized_end=4359,
)
_sym_db.RegisterEnumDescriptor(_TEXTURETILINGYPOSITION)

TextureTilingYPosition = enum_type_wrapper.EnumTypeWrapper(_TEXTURETILINGYPOSITION)
_TEXTURETILINGCONTINUITY = _descriptor.EnumDescriptor(
  name='TextureTilingContinuity',
  full_name='TSCH.TextureTilingContinuity',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='textureTilingContinuityNone', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='textureTilingContinuityGlobal', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='textureTilingContinuitySeries', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='textureTilingContinuityJittered', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=4362,
  serialized_end=4527,
)
_sym_db.RegisterEnumDescriptor(_TEXTURETILINGCONTINUITY)

TextureTilingContinuity = enum_type_wrapper.EnumTypeWrapper(_TEXTURETILINGCONTINUITY)
_FILLPROPERTYTYPE = _descriptor.EnumDescriptor(
  name='FillPropertyType',
  full_name='TSCH.FillPropertyType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='fillPropertyTypeUndefined', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='fillPropertyTypeArea', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='fillPropertyTypeBar', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='fillPropertyTypeColumn', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='fillPropertyTypeLine', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='fillPropertyTypePie', index=5, number=5,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=4530,
  serialized_end=4709,
)
_sym_db.RegisterEnumDescriptor(_FILLPROPERTYTYPE)

FillPropertyType = enum_type_wrapper.EnumTypeWrapper(_FILLPROPERTYTYPE)
textureTilingModeNone = 0
textureTilingModeTallest = 1
textureTilingFaceAll = 0
textureTilingFaceTopAndBottom = 1
textureTilingFaceSide = 2
textureTilingWrapProjected = 0
textureTilingWrapFaceWrap = 1
textureTilingXPositionLeft = 0
textureTilingXPositionCenter = 1
textureTilingXPositionRight = 2
textureTilingYPositionTop = 0
textureTilingYPositionMiddle = 1
textureTilingYPositionBottom = 2
textureTilingContinuityNone = 0
textureTilingContinuityGlobal = 1
textureTilingContinuitySeries = 2
textureTilingContinuityJittered = 3
fillPropertyTypeUndefined = 0
fillPropertyTypeArea = 1
fillPropertyTypeBar = 2
fillPropertyTypeColumn = 3
fillPropertyTypeLine = 4
fillPropertyTypePie = 5

FILL3D_FIELD_NUMBER = 100
fill3d = _descriptor.FieldDescriptor(
  name='fill3d', full_name='TSCH.fill3d', index=0,
  number=100, type=11, cpp_type=10, label=1,
  has_default_value=False, default_value=None,
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  serialized_options=None, file=DESCRIPTOR)


_CHART3DENVIRONMENTPACKAGEARCHIVE = _descriptor.Descriptor(
  name='Chart3DEnvironmentPackageArchive',
  full_name='TSCH.Chart3DEnvironmentPackageArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='materials', full_name='TSCH.Chart3DEnvironmentPackageArchive.materials', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=68,
  serialized_end=162,
)


_CHART3DFILLARCHIVE = _descriptor.Descriptor(
  name='Chart3DFillArchive',
  full_name='TSCH.Chart3DFillArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='lightingmodel', full_name='TSCH.Chart3DFillArchive.lightingmodel', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='textureset_id', full_name='TSCH.Chart3DFillArchive.textureset_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fill_type', full_name='TSCH.Chart3DFillArchive.fill_type', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='series_index', full_name='TSCH.Chart3DFillArchive.series_index', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=165,
  serialized_end=331,
)


_CHART3DPOINTLIGHTARCHIVE = _descriptor.Descriptor(
  name='Chart3DPointLightArchive',
  full_name='TSCH.Chart3DPointLightArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='position', full_name='TSCH.Chart3DPointLightArchive.position', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=333,
  serialized_end=405,
)


_CHART3DDIRECTIONALLIGHTARCHIVE = _descriptor.Descriptor(
  name='Chart3DDirectionalLightArchive',
  full_name='TSCH.Chart3DDirectionalLightArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='direction', full_name='TSCH.Chart3DDirectionalLightArchive.direction', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=407,
  serialized_end=486,
)


_CHART3DSPOTLIGHTARCHIVE = _descriptor.Descriptor(
  name='Chart3DSpotLightArchive',
  full_name='TSCH.Chart3DSpotLightArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='position', full_name='TSCH.Chart3DSpotLightArchive.position', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='direction', full_name='TSCH.Chart3DSpotLightArchive.direction', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cutoff', full_name='TSCH.Chart3DSpotLightArchive.cutoff', index=2,
      number=3, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dropoff', full_name='TSCH.Chart3DSpotLightArchive.dropoff', index=3,
      number=4, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=489,
  serialized_end=640,
)


_CHART3DLIGHTARCHIVE = _descriptor.Descriptor(
  name='Chart3DLightArchive',
  full_name='TSCH.Chart3DLightArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='TSCH.Chart3DLightArchive.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ambient_color', full_name='TSCH.Chart3DLightArchive.ambient_color', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='diffuse_color', full_name='TSCH.Chart3DLightArchive.diffuse_color', index=2,
      number=3, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='specular_color', full_name='TSCH.Chart3DLightArchive.specular_color', index=3,
      number=4, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='intensity', full_name='TSCH.Chart3DLightArchive.intensity', index=4,
      number=5, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attenuation', full_name='TSCH.Chart3DLightArchive.attenuation', index=5,
      number=6, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='coordinate_space', full_name='TSCH.Chart3DLightArchive.coordinate_space', index=6,
      number=7, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='enabled', full_name='TSCH.Chart3DLightArchive.enabled', index=7,
      number=8, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='point_light', full_name='TSCH.Chart3DLightArchive.point_light', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='directional_light', full_name='TSCH.Chart3DLightArchive.directional_light', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='spot_light', full_name='TSCH.Chart3DLightArchive.spot_light', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=643,
  serialized_end=1112,
)


_CHART3DLIGHTINGMODELARCHIVE = _descriptor.Descriptor(
  name='Chart3DLightingModelArchive',
  full_name='TSCH.Chart3DLightingModelArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='phong', full_name='TSCH.Chart3DLightingModelArchive.phong', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fixed_function', full_name='TSCH.Chart3DLightingModelArchive.fixed_function', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='environment', full_name='TSCH.Chart3DLightingModelArchive.environment', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1115,
  serialized_end=1332,
)


_CHART3DLIGHTINGPACKAGEARCHIVE = _descriptor.Descriptor(
  name='Chart3DLightingPackageArchive',
  full_name='TSCH.Chart3DLightingPackageArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='TSCH.Chart3DLightingPackageArchive.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lights', full_name='TSCH.Chart3DLightingPackageArchive.lights', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1334,
  serialized_end=1422,
)


_CHART3DTEXTURESMATERIALARCHIVE = _descriptor.Descriptor(
  name='Chart3DTexturesMaterialArchive',
  full_name='TSCH.Chart3DTexturesMaterialArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='color', full_name='TSCH.Chart3DTexturesMaterialArchive.color', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='textures', full_name='TSCH.Chart3DTexturesMaterialArchive.textures', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1425,
  serialized_end=1559,
)


_CHART3DEMISSIVEMATERIALARCHIVE = _descriptor.Descriptor(
  name='Chart3DEmissiveMaterialArchive',
  full_name='TSCH.Chart3DEmissiveMaterialArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='super', full_name='TSCH.Chart3DEmissiveMaterialArchive.super', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tilings', full_name='TSCH.Chart3DEmissiveMaterialArchive.tilings', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1562,
  serialized_end=1704,
)


_CHART3DDIFFUSEMATERIALARCHIVE = _descriptor.Descriptor(
  name='Chart3DDiffuseMaterialArchive',
  full_name='TSCH.Chart3DDiffuseMaterialArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='super', full_name='TSCH.Chart3DDiffuseMaterialArchive.super', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tilings', full_name='TSCH.Chart3DDiffuseMaterialArchive.tilings', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1707,
  serialized_end=1848,
)


_CHART3DMODULATEMATERIALARCHIVE = _descriptor.Descriptor(
  name='Chart3DModulateMaterialArchive',
  full_name='TSCH.Chart3DModulateMaterialArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='super', full_name='TSCH.Chart3DModulateMaterialArchive.super', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tilings', full_name='TSCH.Chart3DModulateMaterialArchive.tilings', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1851,
  serialized_end=1993,
)


_CHART3DSPECULARMATERIALARCHIVE = _descriptor.Descriptor(
  name='Chart3DSpecularMaterialArchive',
  full_name='TSCH.Chart3DSpecularMaterialArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='super', full_name='TSCH.Chart3DSpecularMaterialArchive.super', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tilings', full_name='TSCH.Chart3DSpecularMaterialArchive.tilings', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1996,
  serialized_end=2138,
)


_CHART3DSHININESSMATERIALARCHIVE = _descriptor.Descriptor(
  name='Chart3DShininessMaterialArchive',
  full_name='TSCH.Chart3DShininessMaterialArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='super', full_name='TSCH.Chart3DShininessMaterialArchive.super', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tilings', full_name='TSCH.Chart3DShininessMaterialArchive.tilings', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2141,
  serialized_end=2284,
)


_CHART3DENVIRONMENTMATERIALARCHIVE = _descriptor.Descriptor(
  name='Chart3DEnvironmentMaterialArchive',
  full_name='TSCH.Chart3DEnvironmentMaterialArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='super', full_name='TSCH.Chart3DEnvironmentMaterialArchive.super', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='OBSOLETE_tilings', full_name='TSCH.Chart3DEnvironmentMaterialArchive.OBSOLETE_tilings', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='decalMode', full_name='TSCH.Chart3DEnvironmentMaterialArchive.decalMode', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tilings', full_name='TSCH.Chart3DEnvironmentMaterialArchive.tilings', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2287,
  serialized_end=2521,
)


_CHART3DFIXEDFUNCTIONLIGHTINGMODELARCHIVE = _descriptor.Descriptor(
  name='Chart3DFixedFunctionLightingModelArchive',
  full_name='TSCH.Chart3DFixedFunctionLightingModelArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='materials', full_name='TSCH.Chart3DFixedFunctionLightingModelArchive.materials', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2523,
  serialized_end=2626,
)


_CHART3DPHONGLIGHTINGMODELARCHIVE = _descriptor.Descriptor(
  name='Chart3DPhongLightingModelArchive',
  full_name='TSCH.Chart3DPhongLightingModelArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='materials', full_name='TSCH.Chart3DPhongLightingModelArchive.materials', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2628,
  serialized_end=2723,
)


_CHART3DPHONGMATERIALPACKAGEARCHIVE = _descriptor.Descriptor(
  name='Chart3DPhongMaterialPackageArchive',
  full_name='TSCH.Chart3DPhongMaterialPackageArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='emissive', full_name='TSCH.Chart3DPhongMaterialPackageArchive.emissive', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='diffuse', full_name='TSCH.Chart3DPhongMaterialPackageArchive.diffuse', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='modulate', full_name='TSCH.Chart3DPhongMaterialPackageArchive.modulate', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='specular', full_name='TSCH.Chart3DPhongMaterialPackageArchive.specular', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='shininess', full_name='TSCH.Chart3DPhongMaterialPackageArchive.shininess', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2726,
  serialized_end=3042,
)


_CHART3DTSPIMAGEDATATEXTUREARCHIVE = _descriptor.Descriptor(
  name='Chart3DTSPImageDataTextureArchive',
  full_name='TSCH.Chart3DTSPImageDataTextureArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='TSCH.Chart3DTSPImageDataTextureArchive.data', index=0,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mipmapdata', full_name='TSCH.Chart3DTSPImageDataTextureArchive.mipmapdata', index=1,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='database_data', full_name='TSCH.Chart3DTSPImageDataTextureArchive.database_data', index=2,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='database_mipmapdata', full_name='TSCH.Chart3DTSPImageDataTextureArchive.database_mipmapdata', index=3,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3045,
  serialized_end=3238,
)


_CHART3DBASEIMAGETEXTURETILINGARCHIVE = _descriptor.Descriptor(
  name='Chart3DBaseImageTextureTilingArchive',
  full_name='TSCH.Chart3DBaseImageTextureTilingArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='scale', full_name='TSCH.Chart3DBaseImageTextureTilingArchive.scale', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rotation', full_name='TSCH.Chart3DBaseImageTextureTilingArchive.rotation', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3240,
  serialized_end=3339,
)


_CHART3DIMAGETEXTURETILINGARCHIVE = _descriptor.Descriptor(
  name='Chart3DImageTextureTilingArchive',
  full_name='TSCH.Chart3DImageTextureTilingArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='super', full_name='TSCH.Chart3DImageTextureTilingArchive.super', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mode', full_name='TSCH.Chart3DImageTextureTilingArchive.mode', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='wrap', full_name='TSCH.Chart3DImageTextureTilingArchive.wrap', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='face', full_name='TSCH.Chart3DImageTextureTilingArchive.face', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='xposition', full_name='TSCH.Chart3DImageTextureTilingArchive.xposition', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='yposition', full_name='TSCH.Chart3DImageTextureTilingArchive.yposition', index=5,
      number=6, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='scontinuity', full_name='TSCH.Chart3DImageTextureTilingArchive.scontinuity', index=6,
      number=7, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tcontinuity', full_name='TSCH.Chart3DImageTextureTilingArchive.tcontinuity', index=7,
      number=8, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='reveal', full_name='TSCH.Chart3DImageTextureTilingArchive.reveal', index=8,
      number=9, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3342,
  serialized_end=3770,
)


_CHART3DVECTORARCHIVE = _descriptor.Descriptor(
  name='Chart3DVectorArchive',
  full_name='TSCH.Chart3DVectorArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='TSCH.Chart3DVectorArchive.x', index=0,
      number=1, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='y', full_name='TSCH.Chart3DVectorArchive.y', index=1,
      number=2, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='z', full_name='TSCH.Chart3DVectorArchive.z', index=2,
      number=3, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='w', full_name='TSCH.Chart3DVectorArchive.w', index=3,
      number=4, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3772,
  serialized_end=3838,
)

_CHART3DENVIRONMENTPACKAGEARCHIVE.fields_by_name['materials'].message_type = _CHART3DENVIRONMENTMATERIALARCHIVE
_CHART3DFILLARCHIVE.fields_by_name['lightingmodel'].message_type = _CHART3DLIGHTINGMODELARCHIVE
_CHART3DFILLARCHIVE.fields_by_name['fill_type'].enum_type = _FILLPROPERTYTYPE
_CHART3DPOINTLIGHTARCHIVE.fields_by_name['position'].message_type = _CHART3DVECTORARCHIVE
_CHART3DDIRECTIONALLIGHTARCHIVE.fields_by_name['direction'].message_type = _CHART3DVECTORARCHIVE
_CHART3DSPOTLIGHTARCHIVE.fields_by_name['position'].message_type = _CHART3DVECTORARCHIVE
_CHART3DSPOTLIGHTARCHIVE.fields_by_name['direction'].message_type = _CHART3DVECTORARCHIVE
_CHART3DLIGHTARCHIVE.fields_by_name['ambient_color'].message_type = _CHART3DVECTORARCHIVE
_CHART3DLIGHTARCHIVE.fields_by_name['diffuse_color'].message_type = _CHART3DVECTORARCHIVE
_CHART3DLIGHTARCHIVE.fields_by_name['specular_color'].message_type = _CHART3DVECTORARCHIVE
_CHART3DLIGHTARCHIVE.fields_by_name['attenuation'].message_type = _CHART3DVECTORARCHIVE
_CHART3DLIGHTARCHIVE.fields_by_name['point_light'].message_type = _CHART3DPOINTLIGHTARCHIVE
_CHART3DLIGHTARCHIVE.fields_by_name['directional_light'].message_type = _CHART3DDIRECTIONALLIGHTARCHIVE
_CHART3DLIGHTARCHIVE.fields_by_name['spot_light'].message_type = _CHART3DSPOTLIGHTARCHIVE
_CHART3DLIGHTINGMODELARCHIVE.fields_by_name['phong'].message_type = _CHART3DPHONGLIGHTINGMODELARCHIVE
_CHART3DLIGHTINGMODELARCHIVE.fields_by_name['fixed_function'].message_type = _CHART3DFIXEDFUNCTIONLIGHTINGMODELARCHIVE
_CHART3DLIGHTINGMODELARCHIVE.fields_by_name['environment'].message_type = _CHART3DENVIRONMENTPACKAGEARCHIVE
_CHART3DLIGHTINGPACKAGEARCHIVE.fields_by_name['lights'].message_type = _CHART3DLIGHTARCHIVE
_CHART3DTEXTURESMATERIALARCHIVE.fields_by_name['color'].message_type = _CHART3DVECTORARCHIVE
_CHART3DTEXTURESMATERIALARCHIVE.fields_by_name['textures'].message_type = _CHART3DTSPIMAGEDATATEXTUREARCHIVE
_CHART3DEMISSIVEMATERIALARCHIVE.fields_by_name['super'].message_type = _CHART3DTEXTURESMATERIALARCHIVE
_CHART3DEMISSIVEMATERIALARCHIVE.fields_by_name['tilings'].message_type = _CHART3DIMAGETEXTURETILINGARCHIVE
_CHART3DDIFFUSEMATERIALARCHIVE.fields_by_name['super'].message_type = _CHART3DTEXTURESMATERIALARCHIVE
_CHART3DDIFFUSEMATERIALARCHIVE.fields_by_name['tilings'].message_type = _CHART3DIMAGETEXTURETILINGARCHIVE
_CHART3DMODULATEMATERIALARCHIVE.fields_by_name['super'].message_type = _CHART3DTEXTURESMATERIALARCHIVE
_CHART3DMODULATEMATERIALARCHIVE.fields_by_name['tilings'].message_type = _CHART3DIMAGETEXTURETILINGARCHIVE
_CHART3DSPECULARMATERIALARCHIVE.fields_by_name['super'].message_type = _CHART3DTEXTURESMATERIALARCHIVE
_CHART3DSPECULARMATERIALARCHIVE.fields_by_name['tilings'].message_type = _CHART3DIMAGETEXTURETILINGARCHIVE
_CHART3DSHININESSMATERIALARCHIVE.fields_by_name['super'].message_type = _CHART3DTEXTURESMATERIALARCHIVE
_CHART3DSHININESSMATERIALARCHIVE.fields_by_name['tilings'].message_type = _CHART3DIMAGETEXTURETILINGARCHIVE
_CHART3DENVIRONMENTMATERIALARCHIVE.fields_by_name['super'].message_type = _CHART3DTEXTURESMATERIALARCHIVE
_CHART3DENVIRONMENTMATERIALARCHIVE.fields_by_name['OBSOLETE_tilings'].message_type = _CHART3DBASEIMAGETEXTURETILINGARCHIVE
_CHART3DENVIRONMENTMATERIALARCHIVE.fields_by_name['tilings'].message_type = _CHART3DIMAGETEXTURETILINGARCHIVE
_CHART3DFIXEDFUNCTIONLIGHTINGMODELARCHIVE.fields_by_name['materials'].message_type = _CHART3DPHONGMATERIALPACKAGEARCHIVE
_CHART3DPHONGLIGHTINGMODELARCHIVE.fields_by_name['materials'].message_type = _CHART3DPHONGMATERIALPACKAGEARCHIVE
_CHART3DPHONGMATERIALPACKAGEARCHIVE.fields_by_name['emissive'].message_type = _CHART3DEMISSIVEMATERIALARCHIVE
_CHART3DPHONGMATERIALPACKAGEARCHIVE.fields_by_name['diffuse'].message_type = _CHART3DDIFFUSEMATERIALARCHIVE
_CHART3DPHONGMATERIALPACKAGEARCHIVE.fields_by_name['modulate'].message_type = _CHART3DMODULATEMATERIALARCHIVE
_CHART3DPHONGMATERIALPACKAGEARCHIVE.fields_by_name['specular'].message_type = _CHART3DSPECULARMATERIALARCHIVE
_CHART3DPHONGMATERIALPACKAGEARCHIVE.fields_by_name['shininess'].message_type = _CHART3DSHININESSMATERIALARCHIVE
_CHART3DTSPIMAGEDATATEXTUREARCHIVE.fields_by_name['data'].message_type = TSPMessages__pb2._DATAREFERENCE
_CHART3DTSPIMAGEDATATEXTUREARCHIVE.fields_by_name['mipmapdata'].message_type = TSPMessages__pb2._DATAREFERENCE
_CHART3DTSPIMAGEDATATEXTUREARCHIVE.fields_by_name['database_data'].message_type = TSPMessages__pb2._REFERENCE
_CHART3DTSPIMAGEDATATEXTUREARCHIVE.fields_by_name['database_mipmapdata'].message_type = TSPMessages__pb2._REFERENCE
_CHART3DBASEIMAGETEXTURETILINGARCHIVE.fields_by_name['scale'].message_type = _CHART3DVECTORARCHIVE
_CHART3DIMAGETEXTURETILINGARCHIVE.fields_by_name['super'].message_type = _CHART3DBASEIMAGETEXTURETILINGARCHIVE
_CHART3DIMAGETEXTURETILINGARCHIVE.fields_by_name['mode'].enum_type = _TEXTURETILINGMODE
_CHART3DIMAGETEXTURETILINGARCHIVE.fields_by_name['wrap'].enum_type = _TEXTURETILINGWRAP
_CHART3DIMAGETEXTURETILINGARCHIVE.fields_by_name['face'].enum_type = _TEXTURETILINGFACE
_CHART3DIMAGETEXTURETILINGARCHIVE.fields_by_name['xposition'].enum_type = _TEXTURETILINGXPOSITION
_CHART3DIMAGETEXTURETILINGARCHIVE.fields_by_name['yposition'].enum_type = _TEXTURETILINGYPOSITION
_CHART3DIMAGETEXTURETILINGARCHIVE.fields_by_name['scontinuity'].enum_type = _TEXTURETILINGCONTINUITY
_CHART3DIMAGETEXTURETILINGARCHIVE.fields_by_name['tcontinuity'].enum_type = _TEXTURETILINGCONTINUITY
DESCRIPTOR.message_types_by_name['Chart3DEnvironmentPackageArchive'] = _CHART3DENVIRONMENTPACKAGEARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DFillArchive'] = _CHART3DFILLARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DPointLightArchive'] = _CHART3DPOINTLIGHTARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DDirectionalLightArchive'] = _CHART3DDIRECTIONALLIGHTARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DSpotLightArchive'] = _CHART3DSPOTLIGHTARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DLightArchive'] = _CHART3DLIGHTARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DLightingModelArchive'] = _CHART3DLIGHTINGMODELARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DLightingPackageArchive'] = _CHART3DLIGHTINGPACKAGEARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DTexturesMaterialArchive'] = _CHART3DTEXTURESMATERIALARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DEmissiveMaterialArchive'] = _CHART3DEMISSIVEMATERIALARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DDiffuseMaterialArchive'] = _CHART3DDIFFUSEMATERIALARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DModulateMaterialArchive'] = _CHART3DMODULATEMATERIALARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DSpecularMaterialArchive'] = _CHART3DSPECULARMATERIALARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DShininessMaterialArchive'] = _CHART3DSHININESSMATERIALARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DEnvironmentMaterialArchive'] = _CHART3DENVIRONMENTMATERIALARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DFixedFunctionLightingModelArchive'] = _CHART3DFIXEDFUNCTIONLIGHTINGMODELARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DPhongLightingModelArchive'] = _CHART3DPHONGLIGHTINGMODELARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DPhongMaterialPackageArchive'] = _CHART3DPHONGMATERIALPACKAGEARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DTSPImageDataTextureArchive'] = _CHART3DTSPIMAGEDATATEXTUREARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DBaseImageTextureTilingArchive'] = _CHART3DBASEIMAGETEXTURETILINGARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DImageTextureTilingArchive'] = _CHART3DIMAGETEXTURETILINGARCHIVE
DESCRIPTOR.message_types_by_name['Chart3DVectorArchive'] = _CHART3DVECTORARCHIVE
DESCRIPTOR.enum_types_by_name['TextureTilingMode'] = _TEXTURETILINGMODE
DESCRIPTOR.enum_types_by_name['TextureTilingFace'] = _TEXTURETILINGFACE
DESCRIPTOR.enum_types_by_name['TextureTilingWrap'] = _TEXTURETILINGWRAP
DESCRIPTOR.enum_types_by_name['TextureTilingXPosition'] = _TEXTURETILINGXPOSITION
DESCRIPTOR.enum_types_by_name['TextureTilingYPosition'] = _TEXTURETILINGYPOSITION
DESCRIPTOR.enum_types_by_name['TextureTilingContinuity'] = _TEXTURETILINGCONTINUITY
DESCRIPTOR.enum_types_by_name['FillPropertyType'] = _FILLPROPERTYTYPE
DESCRIPTOR.extensions_by_name['fill3d'] = fill3d
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Chart3DEnvironmentPackageArchive = _reflection.GeneratedProtocolMessageType('Chart3DEnvironmentPackageArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DENVIRONMENTPACKAGEARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DEnvironmentPackageArchive)
  ))
_sym_db.RegisterMessage(Chart3DEnvironmentPackageArchive)

Chart3DFillArchive = _reflection.GeneratedProtocolMessageType('Chart3DFillArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DFILLARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DFillArchive)
  ))
_sym_db.RegisterMessage(Chart3DFillArchive)

Chart3DPointLightArchive = _reflection.GeneratedProtocolMessageType('Chart3DPointLightArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DPOINTLIGHTARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DPointLightArchive)
  ))
_sym_db.RegisterMessage(Chart3DPointLightArchive)

Chart3DDirectionalLightArchive = _reflection.GeneratedProtocolMessageType('Chart3DDirectionalLightArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DDIRECTIONALLIGHTARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DDirectionalLightArchive)
  ))
_sym_db.RegisterMessage(Chart3DDirectionalLightArchive)

Chart3DSpotLightArchive = _reflection.GeneratedProtocolMessageType('Chart3DSpotLightArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DSPOTLIGHTARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DSpotLightArchive)
  ))
_sym_db.RegisterMessage(Chart3DSpotLightArchive)

Chart3DLightArchive = _reflection.GeneratedProtocolMessageType('Chart3DLightArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DLIGHTARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DLightArchive)
  ))
_sym_db.RegisterMessage(Chart3DLightArchive)

Chart3DLightingModelArchive = _reflection.GeneratedProtocolMessageType('Chart3DLightingModelArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DLIGHTINGMODELARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DLightingModelArchive)
  ))
_sym_db.RegisterMessage(Chart3DLightingModelArchive)

Chart3DLightingPackageArchive = _reflection.GeneratedProtocolMessageType('Chart3DLightingPackageArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DLIGHTINGPACKAGEARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DLightingPackageArchive)
  ))
_sym_db.RegisterMessage(Chart3DLightingPackageArchive)

Chart3DTexturesMaterialArchive = _reflection.GeneratedProtocolMessageType('Chart3DTexturesMaterialArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DTEXTURESMATERIALARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DTexturesMaterialArchive)
  ))
_sym_db.RegisterMessage(Chart3DTexturesMaterialArchive)

Chart3DEmissiveMaterialArchive = _reflection.GeneratedProtocolMessageType('Chart3DEmissiveMaterialArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DEMISSIVEMATERIALARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DEmissiveMaterialArchive)
  ))
_sym_db.RegisterMessage(Chart3DEmissiveMaterialArchive)

Chart3DDiffuseMaterialArchive = _reflection.GeneratedProtocolMessageType('Chart3DDiffuseMaterialArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DDIFFUSEMATERIALARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DDiffuseMaterialArchive)
  ))
_sym_db.RegisterMessage(Chart3DDiffuseMaterialArchive)

Chart3DModulateMaterialArchive = _reflection.GeneratedProtocolMessageType('Chart3DModulateMaterialArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DMODULATEMATERIALARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DModulateMaterialArchive)
  ))
_sym_db.RegisterMessage(Chart3DModulateMaterialArchive)

Chart3DSpecularMaterialArchive = _reflection.GeneratedProtocolMessageType('Chart3DSpecularMaterialArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DSPECULARMATERIALARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DSpecularMaterialArchive)
  ))
_sym_db.RegisterMessage(Chart3DSpecularMaterialArchive)

Chart3DShininessMaterialArchive = _reflection.GeneratedProtocolMessageType('Chart3DShininessMaterialArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DSHININESSMATERIALARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DShininessMaterialArchive)
  ))
_sym_db.RegisterMessage(Chart3DShininessMaterialArchive)

Chart3DEnvironmentMaterialArchive = _reflection.GeneratedProtocolMessageType('Chart3DEnvironmentMaterialArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DENVIRONMENTMATERIALARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DEnvironmentMaterialArchive)
  ))
_sym_db.RegisterMessage(Chart3DEnvironmentMaterialArchive)

Chart3DFixedFunctionLightingModelArchive = _reflection.GeneratedProtocolMessageType('Chart3DFixedFunctionLightingModelArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DFIXEDFUNCTIONLIGHTINGMODELARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DFixedFunctionLightingModelArchive)
  ))
_sym_db.RegisterMessage(Chart3DFixedFunctionLightingModelArchive)

Chart3DPhongLightingModelArchive = _reflection.GeneratedProtocolMessageType('Chart3DPhongLightingModelArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DPHONGLIGHTINGMODELARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DPhongLightingModelArchive)
  ))
_sym_db.RegisterMessage(Chart3DPhongLightingModelArchive)

Chart3DPhongMaterialPackageArchive = _reflection.GeneratedProtocolMessageType('Chart3DPhongMaterialPackageArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DPHONGMATERIALPACKAGEARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DPhongMaterialPackageArchive)
  ))
_sym_db.RegisterMessage(Chart3DPhongMaterialPackageArchive)

Chart3DTSPImageDataTextureArchive = _reflection.GeneratedProtocolMessageType('Chart3DTSPImageDataTextureArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DTSPIMAGEDATATEXTUREARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DTSPImageDataTextureArchive)
  ))
_sym_db.RegisterMessage(Chart3DTSPImageDataTextureArchive)

Chart3DBaseImageTextureTilingArchive = _reflection.GeneratedProtocolMessageType('Chart3DBaseImageTextureTilingArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DBASEIMAGETEXTURETILINGARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DBaseImageTextureTilingArchive)
  ))
_sym_db.RegisterMessage(Chart3DBaseImageTextureTilingArchive)

Chart3DImageTextureTilingArchive = _reflection.GeneratedProtocolMessageType('Chart3DImageTextureTilingArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DIMAGETEXTURETILINGARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DImageTextureTilingArchive)
  ))
_sym_db.RegisterMessage(Chart3DImageTextureTilingArchive)

Chart3DVectorArchive = _reflection.GeneratedProtocolMessageType('Chart3DVectorArchive', (_message.Message,), dict(
  DESCRIPTOR = _CHART3DVECTORARCHIVE,
  __module__ = 'TSCH3DArchives_pb2'
  # @@protoc_insertion_point(class_scope:TSCH.Chart3DVectorArchive)
  ))
_sym_db.RegisterMessage(Chart3DVectorArchive)

fill3d.message_type = _CHART3DFILLARCHIVE
TSDArchives__pb2.FillArchive.RegisterExtension(fill3d)

# @@protoc_insertion_point(module_scope)



from __future__ import print_function
from __future__ import absolute_import
import argparse

import keynote_parser

keynote_parser.__command_line_invocation__ = True  # noqa

from keynote_parser import __version__, __supported_keynote_version__
from collections import Counter
from .bundle_utils import warn_once_on_newer_keynote, get_installed_keynote_version
from .file_utils import process
from .replacement import Replacement, parse_json


def parse_replacements(**kwargs):
    json_filename = kwargs.get('replacements')
    if json_filename:
        return parse_json(json_filename)
    else:
        return []


def unpack_command(input, output=None, **kwargs):
    process(input, output or input.replace('.key', ''), replacements=parse_replacements(**kwargs))


def pack_command(input, output=None, **kwargs):
    process(input, output or (input + ".key"), replacements=parse_replacements(**kwargs))


def ls_command(input, **kwargs):
    process(input, '-')


def cat_command(input, filename, **kwargs):
    process(
        input,
        '-',
        subfile=filename,
        replacements=parse_replacements(**kwargs),
        raw=kwargs.get('raw'),
    )


def replace_command(input, **kwargs):
    output = kwargs.get('output', None) or input
    replacements = parse_replacements(**kwargs)
    find, _replace = kwargs.get('find'), kwargs.get('replace')
    if find and _replace:
        replacements.append(Replacement(find, _replace))
    if not replacements:
        print("WARNING: No replacements passed. No change.")
        return
    for ((old, new), count) in list(Counter(process(input, output, replacements)).items()):
        if count == 1:
            print("Replaced %s with %s." % (repr(old), repr(new)))
        else:
            print("Replaced %s with %s %d times." % (repr(old), repr(new), count))


def add_replacement_arg(parser):
    parser.add_argument("--replacements", help="apply replacements from a json or yaml file")


def main():
    installed_version = get_installed_keynote_version()
    install_warning = ""
    if installed_version and __supported_keynote_version__ < installed_version:
        install_warning = (
            " (Installed Keynote version %s not yet supported.)" % installed_version
        )
    parser = argparse.ArgumentParser(
        description=(
            "manipulate Apple Keynote .key files. version %s, supports Keynote versions up to %s.%s"
        )
        % (__version__, __supported_keynote_version__, install_warning)
    )
    parser.add_argument('-v', '--version', action='version', version=__version__)

    subparsers = parser.add_subparsers()

    parser_unpack = subparsers.add_parser('unpack')
    parser_unpack.add_argument("input", help="a .key file")
    parser_unpack.add_argument("--output", "-o", help="a directory name to unpack into")
    add_replacement_arg(parser_unpack)
    parser_unpack.set_defaults(func=unpack_command)

    parser_pack = subparsers.add_parser('pack')
    parser_pack.add_argument("input", help="a directory of an unpacked .key file")
    parser_pack.add_argument("--output", "-o", help="a keynote file name to unpack into")
    add_replacement_arg(parser_pack)
    parser_pack.set_defaults(func=pack_command)

    parser_ls = subparsers.add_parser('ls')
    parser_ls.add_argument("input", help="a .key file")
    parser_ls.set_defaults(func=ls_command)

    parser_cat = subparsers.add_parser('cat')
    parser_cat.add_argument("input", help="a .key file")
    parser_cat.add_argument(
        "filename", help="a file within that .key file to cat, decoding .iwa to .yaml"
    )
    parser_cat.add_argument(
        "--raw", action='store_true', help="always return the original file with no decoding"
    )
    add_replacement_arg(parser_cat)
    parser_cat.set_defaults(func=cat_command)

    parser_replace = subparsers.add_parser('replace')
    parser_replace.add_argument("input", help="a .key file")
    parser_replace.add_argument("--output", "-o", help="a .key file to output to")
    parser_replace.add_argument("--find", help="a pattern to search for in text")
    parser_replace.add_argument("--replace", help="a string to replace with")
    add_replacement_arg(parser_replace)
    parser_replace.set_defaults(func=replace_command)

    args = parser.parse_args()
    if hasattr(args, 'func'):
        warn_once_on_newer_keynote()
        args.func(**vars(args))
    else:
        parser.print_help()
        print()
        warn_once_on_newer_keynote()


if __name__ == "__main__":
    main()
